import { Deferred } from "ts-deferred";
import { ParserRuleContext } from 'antlr4ts';
import { RPScriptListener } from './grammar/RPScriptListener';
import { SymbolContext, VariableContext, LiteralContext, ObjectLiteralContext, ProgramContext, PipeActionsContext, SingleExpressionContext, StatementContext, ActionContext, ArrayLiteralContext, PropertyAssignmentContext, ShortFnContext } from './grammar/RPScriptParser';
import { ParseTreeProperty } from 'antlr4ts/tree';
import { RPScriptParser } from '../antlr/grammar/RPScriptParser';
import { KeywordsMgr } from '../core/keywordsmgr';
export interface TranspileContent {
    mainContent?: string;
    fnContent?: string;
    fullContent?: string;
    verbs?: string[];
}
export interface IncludeContent {
    dir: string;
    rpsContent: string;
    tsContent?: string;
}
export declare class RpsTranspileListener implements RPScriptListener {
    readonly globalEventDeclare: string;
    readonly mainSectionStart: string;
    readonly mainSectionEnd: string;
    readonly runSect: string;
    logger: any;
    deferred: Deferred<any>;
    filepath: string;
    parseTreeProperty: ParseTreeProperty<string>;
    scope: string;
    content: TranspileContent;
    includeContent: IncludeContent[];
    parser: RPScriptParser;
    keywordMgr: KeywordsMgr;
    skipKeywordCheck: boolean;
    constructor(defer: Deferred<any>, filepath: string, parser: RPScriptParser, skipKeywordCheck?: boolean);
    enterProgram(ctx: ProgramContext): void;
    exitProgram(ctx: ProgramContext): void;
    exitPipeActions(ctx: PipeActionsContext): void;
    exitStatement(ctx: StatementContext): void;
    enterAction(ctx: ActionContext): void;
    exitAction(ctx: ActionContext): void;
    exitShortFn(ctx: ShortFnContext): void;
    enterLiteral(ctx: LiteralContext): void;
    enterVariable(ctx: VariableContext): void;
    enterSymbol(ctx: SymbolContext): void;
    exitSingleExpression(ctx: SingleExpressionContext): void;
    exitArrayLiteral(ctx: ArrayLiteralContext): void;
    exitObjectLiteral(ctx: ObjectLiteralContext): void;
    exitPropertyAssignment(ctx: PropertyAssignmentContext): void;
    /**************************************************************/
    private hasActionParent(ctx);
    private hasPipeParent(ctx);
    private hasShortFnParent(ctx);
    hasParent(ctx: ParserRuleContext, parentName: string): boolean;
    private parseOpt(opts);
    private parseVar(ctx);
    private appendToScope(content);
}
