"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const R = __importStar(require("../../lib/ramda.min"));
const tree_1 = require("antlr4ts/tree");
const keywordsmgr_1 = require("../core/keywordsmgr");
const InvalidKeywordException_1 = require("./InvalidKeywordException");
class RpsTranspileListener {
    constructor(defer, filepath, parser, skipKeywordCheck) {
        this.globalEventDeclare = `module.exports = new EventEmitter();\n`;
        this.mainSectionStart = `async function main(){
    module.exports.emit('runner.start');\n
`;
        this.mainSectionEnd = `
    module.exports.emit('runner.end');
}`;
        this.runSect = `

$CONTEXT.event.on ('action', (...params) => {
    let evt = params[2];
    if(evt === 'end') $CONTEXT.$RESULT = params[3];

    module.exports.emit('action',params);
});
setTimeout(main, 100);
`;
        this.deferred = defer;
        // this.logger = Logger.getInstance();
        this.filepath = filepath;
        this.parseTreeProperty = new tree_1.ParseTreeProperty();
        this.scope = "root";
        this.parser = parser;
        this.content = {
            mainContent: "", fullContent: "", fnContent: "", verbs: []
        };
        this.includeContent = [];
        this.keywordMgr = new keywordsmgr_1.KeywordsMgr;
        this.skipKeywordCheck = skipKeywordCheck;
    }
    enterProgram(ctx) {
    }
    exitProgram(ctx) {
        if (ctx.exception)
            throw ctx.exception;
        // if(ctx.exception) this.deferred.reject(ctx.exception)
        else {
            this.content.fullContent += this.globalEventDeclare;
            this.content.fullContent += this.mainSectionStart;
            this.content.fullContent += this.content.mainContent;
            this.content.fullContent += this.mainSectionEnd;
            this.content.fullContent += this.content.fnContent;
            this.content.fullContent += this.runSect;
            this.deferred.resolve(this.content);
        }
    }
    exitPipeActions(ctx) {
        let firstActionContext = this.parseTreeProperty.get(ctx.action()[0]);
        let secActionContext = this.parseTreeProperty.get(ctx.action()[1]);
        let content = '\t' + secActionContext.replace(')', ', ' + firstActionContext) + ');\n';
        if (this.hasActionParent(ctx))
            this.parseTreeProperty.set(ctx, content);
        else
            this.appendToScope(content);
    }
    exitStatement(ctx) {
        if (!!ctx.singleAction()) {
            let action = this.parseTreeProperty.get(ctx.singleAction().action());
            this.parseTreeProperty.set(ctx, action);
        }
        else if (!!ctx.pipeActions()) {
            let context = this.parseTreeProperty.get(ctx.pipeActions());
            this.parseTreeProperty.set(ctx, context);
        }
    }
    enterAction(ctx) {
        if (!this.skipKeywordCheck) {
            if (!this.keywordMgr.isValidKeyword(ctx.WORD().text))
                throw new InvalidKeywordException_1.InvalidKeywordException(ctx);
        }
        this.content.verbs.push(ctx.WORD().text.trim());
        this.parseTreeProperty.set(ctx, `\t${ctx.WORD().text}`);
    }
    exitAction(ctx) {
        let keyword = this.parseTreeProperty.get(ctx);
        let pList = R.map(param => {
            return this.parseTreeProperty.get(param.getChild(0));
        }, ctx.paramList().param());
        let opt = this.parseOpt(ctx.optList().opt()), joinList = pList.join(' , ');
        if (joinList)
            this.parseTreeProperty.set(ctx, `api("${keyword.trim()}" , $CONTEXT , ${opt} , ${joinList})`);
        else
            this.parseTreeProperty.set(ctx, `api("${keyword.trim()}" , $CONTEXT , ${opt} )`);
        if (!this.hasActionParent(ctx)) {
            let content = "\t" + this.parseTreeProperty.get(ctx) + ";\n";
            if (this.hasPipeParent(ctx)) { } //skip
            else
                this.content.mainContent += content;
        }
    }
    exitShortFn(ctx) {
        let params = ctx.variable().map(x => x.text);
        let val = '(' + params.join(',') + ') => ' + this.parseTreeProperty.get(ctx.action());
        this.parseTreeProperty.set(ctx, val);
    }
    // literal | variable | anonFn | symbol | action;
    enterLiteral(ctx) {
        let val = ctx.text;
        if (ctx.TemplateStringLiteral()) {
            val = val.replace(new RegExp('[$]RESULT', 'g'), '$CONTEXT.$RESULT');
        }
        if (ctx.EnvVarLiteral()) {
            val = val.substr(2);
            val = '$CONTEXT.env[' + val + ']';
        }
        this.parseTreeProperty.set(ctx, `${val}`);
    }
    enterVariable(ctx) {
        let variable = this.parseVar(ctx);
        this.parseTreeProperty.set(ctx, `${variable}`);
    }
    enterSymbol(ctx) {
        var lower = ctx.text.charAt(0).toLowerCase() + ctx.text.substr(1);
        this.parseTreeProperty.set(ctx, `Symbol('${lower}')`);
    }
    exitSingleExpression(ctx) {
        if (ctx.variable && ctx.variable())
            this.parseTreeProperty.set(ctx, this.parseTreeProperty.get(ctx.variable()));
        else if (ctx.literal && ctx.literal()) {
            let template = ctx.literal().TemplateStringLiteral(), env = ctx.literal().EnvVarLiteral();
            if (template || env)
                this.parseTreeProperty.set(ctx, this.parseTreeProperty.get(ctx.literal()));
            else
                this.parseTreeProperty.set(ctx, ctx.text);
        }
        else if (ctx.action && ctx.action()) {
            this.parseTreeProperty.set(ctx, this.parseTreeProperty.get(ctx.action()));
        }
        else if (ctx.arrayLiteral && ctx.arrayLiteral()) {
            this.parseTreeProperty.set(ctx, this.parseTreeProperty.get(ctx.arrayLiteral()));
        }
        else if (ctx.objectLiteral && ctx.objectLiteral()) {
            this.parseTreeProperty.set(ctx, this.parseTreeProperty.get(ctx.objectLiteral()));
        }
        else if (ctx.shortFn && ctx.shortFn()) {
            this.parseTreeProperty.set(ctx, this.parseTreeProperty.get(ctx.shortFn()));
        }
        else
            this.parseTreeProperty.set(ctx, ctx.text);
    }
    exitArrayLiteral(ctx) {
        //action , variable
        let array = [];
        for (var i = 0; i < ctx.elementList().singleExpression().length; i++) {
            var expr = ctx.elementList().singleExpression()[i];
            if (expr.action())
                array.push(this.parseTreeProperty.get(expr.action()));
            else if (expr.variable())
                array.push(this.parseTreeProperty.get(expr.variable()));
            else
                array.push(expr.text);
        }
        let arString = '[' + array.join(',') + ']';
        this.parseTreeProperty.set(ctx, arString);
    }
    exitObjectLiteral(ctx) {
        //action , variable
        let array = [];
        for (var i = 0; i < ctx.propertyAssignment().length; i++) {
            var expr = ctx.propertyAssignment()[i];
            array.push(this.parseTreeProperty.get(expr));
        }
        let arString = '{' + array.join(',') + '}';
        this.parseTreeProperty.set(ctx, arString);
    }
    exitPropertyAssignment(ctx) {
        let propName = ctx.propertyName().text;
        let expression = ctx.singleExpression();
        //@ts-ignore
        let expr = this.parseTreeProperty.get(expression);
        let pair = propName + ':' + expr;
        this.parseTreeProperty.set(ctx, pair);
    }
    /**************************************************************/
    hasActionParent(ctx) {
        return this.hasParent(ctx, 'ActionContext');
    }
    hasPipeParent(ctx) {
        return this.hasParent(ctx, 'PipeActionsContext');
    }
    hasShortFnParent(ctx) {
        return this.hasParent(ctx, 'ShortFnContext');
    }
    hasParent(ctx, parentName) {
        let ctxTemp = ctx.parent;
        let isFnParent = false;
        while (ctxTemp) {
            if (parentName === ctxTemp.constructor.name) {
                isFnParent = true;
                break;
            }
            ctxTemp = ctxTemp.parent;
        }
        return isFnParent;
    }
    parseOpt(opts) {
        let obj = {};
        R.forEach(x => {
            let text = eval(x.literal().text);
            obj[x.optName().text] = text;
        }, opts);
        return JSON.stringify(obj);
    }
    parseVar(ctx) {
        let variable = ctx.text;
        // let parsedVar = "typeof "+variable+" != 'undefined' ? " + variable + " : $CONTEXT.variables."+variable;
        let parsedVar = this.hasShortFnParent(ctx) ? variable : "$CONTEXT.variables." + variable;
        if (ctx.text.trim().startsWith('$RESULT'))
            variable = '$CONTEXT.' + variable;
        else
            variable = parsedVar;
        return variable;
    }
    appendToScope(content) {
        if (this.scope === 'root')
            this.content.mainContent += content;
        else
            this.content.fnContent += content;
    }
}
exports.RpsTranspileListener = RpsTranspileListener;
