"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const configstore_1 = __importDefault(require("configstore"));
const R = __importStar(require("../../lib/ramda.min"));
class KeywordsMgr {
    constructor() {
        this.CONFIG_NAME = "rpscript";
        this.configStore = new configstore_1.default(this.CONFIG_NAME);
        let deflt = this.configStore.get('$DEFAULT');
        if (!deflt)
            this.configStore.set('$DEFAULT', {});
    }
    saveModuleKeywords(moduleName, modClazz) {
        let mod = this.configStore.get(moduleName);
        let actionConfigs = this.extractClazzActions(modClazz, moduleName);
        this.updateModule(mod, actionConfigs);
        this.updateDefaults(moduleName, actionConfigs);
    }
    updateModule(mod, actionConfigs) {
        mod.actions = {};
        for (let index in actionConfigs) {
            let config = actionConfigs[index];
            mod.actions[config.methodName] = actionConfigs[index];
        }
        this.configStore.set(mod.name, mod);
    }
    updateDefaults(moduleName, clazzActions) {
        let deflt = this.configStore.get('$DEFAULT');
        let actionsWithDefault = R.filter(config => !!config.verbName, clazzActions);
        actionsWithDefault.forEach(action => {
            let verbName = action.verbName;
            if (deflt[verbName]) {
                let newList = R.filter(a => a['moduleName'] !== action.moduleName, deflt[verbName]);
                newList.push(action);
                deflt[verbName] = newList;
            }
            else
                deflt[verbName] = [action];
        });
        this.configStore.set('$DEFAULT', deflt);
    }
    // iterate keywords
    // filter away action list match moduleName
    removeModuleDefaults(moduleName) {
        let deflts = this.configStore.get('$DEFAULT');
        R.forEachObjIndexed((vals, key) => {
            let output = R.filter(action => action.moduleName !== moduleName, vals);
            if (output.length == 0)
                delete deflts[key];
            else
                deflts[key] = output;
        }, deflts);
        this.configStore.set('$DEFAULT', deflts);
    }
    getDefaultActions() {
        return this.configStore.get('$DEFAULT');
    }
    selectBestFitByPriority(settings) {
        let sort = R.sortBy(R.prop('priority'));
        return sort(settings)[settings.length - 1];
    }
    extractClazzActions(modClazz, moduleName) {
        let actionConfigs = [];
        let methods = Object.getOwnPropertyNames(modClazz.prototype);
        for (var i in methods) {
            let availConfig = modClazz.prototype[methods[i]]['rpsActionConfig'];
            if (availConfig) {
                availConfig.moduleName = moduleName;
                actionConfigs.push(availConfig);
            }
        }
        return actionConfigs;
    }
    isValidKeyword(keyword) {
        let moduleName = '', word = '';
        if (keyword.split('.').length === 2) {
            moduleName = keyword.split('.')[0];
            word = keyword.split('.')[1];
        }
        else
            word = keyword;
        if (moduleName) {
            let mod = this.configStore.get(moduleName);
            return !!mod.actions[word];
        }
        else {
            let deflt = this.configStore.get('$DEFAULT');
            return !!deflt[word];
        }
    }
    getKeywordsModules(keywords) {
        let modules = new Array();
        let keywordList = this.getDefaultActions();
        let output = R.map(k => {
            return R.pluck('moduleName', keywordList[k]);
        }, R.uniq(keywords));
        output = R.uniq(R.flatten(output));
        return R.uniq(output);
    }
}
exports.KeywordsMgr = KeywordsMgr;
