"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const configstore_1 = __importDefault(require("configstore"));
const R = __importStar(require("../../lib/ramda.min"));
const keywordsmgr_1 = require("./keywordsmgr");
const npmMod_1 = require("../helper/npmMod");
const events_1 = require("events");
const deasync_promise_1 = __importDefault(require("deasync-promise"));
const HOMEDIR = require('os').homedir();
class ModuleMgr {
    constructor() {
        this.CONFIG_NAME = "rpscript";
        this.NPM_MOD_PREFIX = "rpscript-api-";
        this.NPM_MOD_ORG_PREFIX = "@typecasting/rpscript-api-";
        this.configStore = new configstore_1.default(this.CONFIG_NAME);
        this.wordMgr = new keywordsmgr_1.KeywordsMgr;
        this.event = new events_1.EventEmitter;
    }
    installModule(npmModuleName, allowExtModule) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let prefix = allowExtModule ? this.NPM_MOD_PREFIX : this.NPM_MOD_ORG_PREFIX;
                if (!npmModuleName.trim().startsWith(prefix))
                    npmModuleName = prefix + npmModuleName.trim();
                let installedInfo = yield this.installFromNpm(npmModuleName);
                this.event.emit(ModuleMgr.MOD_INSTALLED_NPM_EVT, installedInfo);
                //save module info
                this.configStore.set(installedInfo['name'], {
                    name: installedInfo['name'],
                    npmModuleName: installedInfo['npmModuleName'],
                    npmVersion: installedInfo['version'],
                    enabled: true
                });
                //save keywords info
                this.wordMgr.saveModuleKeywords(installedInfo['name'], installedInfo['clazz']);
                let modInfo = this.configStore.get(installedInfo['name']);
                this.event.emit(ModuleMgr.MOD_INSTALLED_CONFIG_EVT, modInfo);
                return modInfo;
            }
            catch (err) {
                this.event.emit(ModuleMgr.MOD_INSTALLED_ERROR_EVT, err);
                throw err;
            }
        });
    }
    installFromNpm(npmModuleName) {
        return __awaiter(this, void 0, void 0, function* () {
            let result = npmMod_1.NpmModHelper.installNpmModule(npmModuleName);
            const mod = yield Promise.resolve().then(() => __importStar(require(`${HOMEDIR}/.rpscript/modules/node_modules/${result.moduleName}`)));
            let modClazz = mod.default;
            let moduleName = modClazz['rpsModuleName'];
            return { clazz: modClazz, name: moduleName, version: result.version, npm: result, npmModuleName: result.moduleName };
        });
    }
    removeModule(moduleName) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let npmModuleName = this.getModuleNpmName(moduleName);
                let removeInfo = npmMod_1.NpmModHelper.removeNpmModule(npmModuleName);
                this.event.emit(ModuleMgr.MOD_REMOVED_NPM_EVT, removeInfo);
                let removedMod = this.removeModuleConfig(npmModuleName);
                this.wordMgr.removeModuleDefaults(removedMod['name']);
                this.event.emit(ModuleMgr.MOD_REMOVED_CONFIG_EVT, removedMod);
            }
            catch (err) {
                this.event.emit(ModuleMgr.MOD_REMOVED_ERROR_EVT, err);
                throw err;
            }
        });
    }
    removeModuleConfig(npmModuleName) {
        let all = this.configStore.all;
        let output = R.values(all);
        //npm name and module name assume to be different
        let modConfig = R.find(R.propEq('npmModuleName', npmModuleName), output);
        this.configStore.delete(modConfig['name']);
        return modConfig;
    }
    listModuleNames() { return R.keys(this.configStore.all); }
    listInstalledModules() { return this.configStore.all; }
    listAvailableModules() { return {}; }
    loadModuleObjs(rpsContext, modules) {
        return __awaiter(this, void 0, void 0, function* () {
            let allModules = this.configStore.all;
            let moduleNames = R.filter(m => m !== '$DEFAULT', R.keys(allModules));
            let moduleObj = {};
            for (let i = 0; i < moduleNames.length; i++) {
                let module = allModules[moduleNames[i]];
                let moduleName = module.npmModuleName;
                let skip = modules ? !R.any(R.identical(moduleNames[i]), modules) : false;
                if (module.enabled && !skip) {
                    let mod = yield Promise.resolve().then(() => __importStar(require(`${HOMEDIR}/.rpscript/modules/node_modules/${moduleName}`)));
                    moduleObj[moduleNames[i]] = new mod.default(rpsContext);
                    this.event.emit(ModuleMgr.MOD_LOADED_EVT, moduleNames[i]);
                }
                else {
                    this.event.emit(ModuleMgr.MOD_DISABLED_EVT, moduleNames[i]);
                }
            }
            moduleObj['api'] = this.genDefaultApi(moduleObj, rpsContext);
            return moduleObj;
        });
    }
    // api(keyword, $CONTEXT , {} , "12121");
    genDefaultApi(modObj, rpsContext) {
        return (keyword, ctx, opt, ...params) => {
            let actions = rpsContext.getRuntimeDefault()[keyword];
            let bestFit = this.wordMgr.selectBestFitByPriority(actions);
            let args = [ctx, opt].concat(params);
            let moduleUse = bestFit.moduleName;
            if (opt['module'])
                moduleUse = opt['module'];
            let promise = modObj[moduleUse][bestFit.methodName].apply(this, args);
            return deasync_promise_1.default(promise);
        };
    }
    getModuleNpmName(moduleName) {
        let npmName = this.configStore.all[moduleName]['npmModuleName'];
        return npmName;
    }
}
ModuleMgr.MOD_INSTALLED_NPM_EVT = "runner.module.installed.npm";
ModuleMgr.MOD_INSTALLED_CONFIG_EVT = "runner.module.installed.config";
ModuleMgr.MOD_INSTALLED_ERROR_EVT = "runner.module.installed.error";
ModuleMgr.MOD_REMOVED_NPM_EVT = "runner.module.removed.npm";
ModuleMgr.MOD_REMOVED_CONFIG_EVT = "runner.module.removed.config";
ModuleMgr.MOD_REMOVED_ERROR_EVT = "runner.module.removed.error";
ModuleMgr.MOD_LOADED_EVT = "runner.module.loaded";
ModuleMgr.MOD_DISABLED_EVT = "runner.module.disabled";
ModuleMgr.MOD_LOAD_ERROR_EVT = "runner.module.load.error";
exports.ModuleMgr = ModuleMgr;
