/// <reference types="node" />
import { EventEmitter } from 'events';
import { RpsContext } from 'rpscript-interface';
import { TranspileContent } from '../antlr/RpsListener';
import { KeywordsMgr } from '..';
export interface RpsMainConfig {
    outputDir?: string;
    skipLinting?: boolean;
    skipOutputTS?: boolean;
    skipRun?: boolean;
    modules?: string[];
    configFilesLocation?: string;
    skipKeywordCheck?: boolean;
}
export interface ExecResult {
    transpile?: TranspileContent;
}
export declare class Runner extends EventEmitter {
    static readonly COMPILE_START_EVT: string;
    static readonly TRANSPILE_EVT: string;
    static readonly TRANSPILE_ERR_EVT: string;
    static readonly LINT_EVT: string;
    static readonly COMPILED_EVT: string;
    static readonly MOD_LOADED_EVT: string;
    static readonly MOD_DISABLED_EVT: string;
    static readonly START_EVT: string;
    static readonly END_EVT: string;
    static readonly ACTION_EVT: string;
    static readonly CTX_PRIOR_SET_EVT: string;
    config: RpsMainConfig;
    runnerListener: EventEmitter;
    wordMgr: KeywordsMgr;
    constructor(config: RpsMainConfig);
    execute(filepath: string, args: any[], content?: string): Promise<ExecResult>;
    exeFromSource(fileName: string): Promise<void>;
    private transpile(filepath, filecontent);
    private run(tsContent, verbs, args?);
    setupModulesContext(ctx: RpsContext): void;
    initializeContext(verbs: string[], args?: any[]): Promise<Object>;
    determineModulesToLoad(verbs?: string[]): string[];
    initRpsContext(configLoc: string, args: any[]): RpsContext;
    convertToTS(filepath: string, content: string): Promise<TranspileContent>;
    private setupLexer(content);
    private setupParser(lexer);
}
